################################################################################
#
#                        G2I : main functions     (V.5)
#
################################################################################


# Rd
# description >> predict G2I groups, based on breast cancer CGHarray GNL profiles   
# argument
# item >> gnldata >> data.frame with rows = genomic targets (clones or probes) and  columns = samples
# item >> cghannot >> data.frame giving target genomic position ; row= genomic targets (clones or probes) ;columns = genomic postion variables 
# item >> sampleannot >> data.frame with prognosis info (such as relapse and delay of relapse)
# item >> n >>  number of genomic targets to be removed at telomeric and centromeric extremities (default value should work)
# item >> chrom >> column name in sampleannot corresponding to the chromosome  with character values (in 1:22 X Y) of genomic targets (clones or probes)
# item >> pos >>  column name in sampleannot corresponding to the mean position in bp of genomic targets (clones or probes)
# item >> startPos >> column name in sampleannot corresponding to the start position in bp of genomic targets (clones or probes)
# item >> endPos >>   column name in sampleannot corresponding to the end position in bp of genomic targets (clones or probes)
# item >> chromNum >> column name in sampleannot corresponding to the chromosome  with numeric values (in 1:24, 23<->X,24 <->Y) of genomic targets (clones or probes)
# item >> chromArm >>  column name in sampleannot corresponding to the chromosome+arm position (ex. 1p, 1q, ...) of genomic targets (clones or probes)
# value >> a list with 1st element = a data.frame as sampleannot but with additional columns (NB, TXP and G2I) -whith G2I column giving genomic groups- and following elements corresponding to the parameters of the decision rule 
# author >> Aurelien de Reynies, Mickael Guedj
# keyword >> methods
# end 
g2i.pretreat <-  function( gnldata=NULL, 
                           cghannot=NULL, 
                           sampleannot=NULL, 
                           n = 5,
                           chrom = "chrom", 
                           pos = "meanPos", 
                           startPos = "start", 
                           endPos = "end", 
                           chromNum = "chrNum",
                           chromArm="arm"){
                                  
      clonesOk  <- intersect(dimnames(cghannot)[[1]],dimnames(gnldata)[[1]])
      samplesOk <- intersect(dimnames(sampleannot)[[1]],dimnames(gnldata)[[2]])
      
      # step 1 >> remove centromeric and telomeric clones
      print("step 1")
      if (n>0){
           clonesOk <- removeCentromericAndTelomericClones(gnldata,cghannot,n,chrom=chrom,pos=pos,startPos=startPos,endPos=endPos,chromNum=chromNum)
      }
      print(length(clonesOk))    
      
      # Ensure that rows and columns are in the same order for the 3 matrix 
      gnldata     <- gnldata[clonesOk,samplesOk]
      cghannot    <- cghannot[clonesOk,]  
      sampleannot <- sampleannot[samplesOk,]
    
      # step 2 >>  gain+loss rate per chromosme arm and then averaged over the genome 
      print("step 2")
      sampleannot$TXP <- gainOrLossRate(gnldata=gnldata,cghannot=cghannot,chromArm=chromArm)
     
      
      # step 3 >> nb regions per local score per chromosome arm, then nb regions over the genome 
      print("step 3")
      sampleannot$NB <- nbNonDiploidRegions(gnldata=gnldata,cghannot=cghannot,chromArm=chromArm)
      
      sampleannot

}


# Rd
# description >> predict G2I groups, based on breast cancer CGHarray GNL profiles   
# argument
# item >> g2iobject >>  list resulting from a call to g2i.learn 
# item >> sampleannot >> data.frame where rows =samples from a validation dataset with prognosis info (such as relapse and delay of relapse) as well as columns NB and TXP resulting from a call to g2i.pretreat
# item >> quantnorm >> boolean (default = TRUE), should quantile normalization be performed for NB and TXP in the new dataset (with baseline = values frome the training dataset)  - set to FALSE when applying the predictor to the training set itself
# value >> a list with 1st element = a data.frame as sampleannot but with additional columns (NB, TXP and G2I) -whith G2I column giving genomic groups- and following elements corresponding to the parameters of the decision rule 
# author >> Aurelien de Reynies, Mickael Guedj
# keyword >> methods
# end 
g2i.predict <- function(  g2iobject=NULL, sampleannot=NULL,quantnorm=TRUE  ){

      sampleannotModif <-sampleannot    
                   
      if(quantnorm){
          sampleannotModif <- g2i.quantNormNBTXP (g2iobject$sampleannot,sampleannot) 
      }
                                   
      res <- g2i.subpredict (g2iobject=g2iobject,  sampleannot=sampleannotModif)
      
      sampleannot$G2I <- res$sampleannot[rownames(sampleannot),"G2I"]
      
      res <- g2iobject
      
      res[[1]] <- sampleannot
      
      if(quantnorm){
          wG3 <- which(sampleannot$G2I==3) 
          wG1 <- which(sampleannot$G2I==1)
          nb13 <- c(max(sampleannot[wG1,"NB"]),min(sampleannot[wG3,"NB"])-1)
          txp13 <- c(max(sampleannot[wG1,"TXP"]),min(sampleannot[wG3,"TXP"])-.004) 
          
          res$"nb1 value on training set"  <- res$nb1
          res$"nb3 value on training set"  <- res$nb3
          res$"txp1 value on training set" <- res$txp1
          res$"txp3 value on training set" <- res$txp3 
          
          res$"nb1"  <- nb13[1]
          res$"nb3"  <- nb13[2]
          res$"txp1" <- txp13[1]
          res$"txp3" <- txp13[2]  
          attr(res$"nb1","info") <- "value in the validation set"  
          attr(res$"nb3","info") <- "value in the validation set" 
          attr(res$"txp1","info") <- "value in the validation set" 
          attr(res$"txp3","info") <- "value in the validation set"       
      }

   
      res
}                                                    

# Rd
# description >> learn a decision rule based on breast cancer CGHarray GNL profiles to discriminate prognosis  
# argument
# item >> sampleannot >> data.frame with prognosis info (such as relapse and delay of relapse) as well as columns NB and TXP resulting from a call to g2i.pretreat 
# item >> method >> character string = method used to determine thresholds , NB: global is slower than local
# item >> doplot >> boolean -> should results be plotted ?
# item >> ratio >>  ...
# item >> event >> ...
# item >> delay >>   ...
# value >> a list with 1st element = a data.frame as sampleannot but with additional columns (NB, TXP and G2I) -whith G2I column giving genomic groups- and following elements corresponding to the parameters of the decision rule 
# author >> Aurelien de Reynies, Mickael Guedj
# keyword >> methods
# end 
g2i.learn  <-  function( sampleannot=NULL,
                         method=c("local","global")[1],                                  
                         doplot = TRUE,                                                   
                         ratio = 0.5,                        
                         event="relapse",
                         delay="delay"){
                                 
        g2iobject <- list(sampleannot=sampleannot)
      
        # step 4 >> get thresholds for group 3 (bad prognosis)
        print("step 4")
        
        if(method=="global"){
            tmp <- g2i.getBestCutsG3(sampleannot,event=event,delay=delay,ratio=ratio)  
            w<-which.min(tmp[,4])
            g2iobject$nb3     <- as.numeric(tmp[w,"thresh.NB"])
            g2iobject$txp3    <- as.numeric(tmp[w,"thresh.TXP"])
        }else{
            tmp   <- g2i.learnThresholdsGroup3(sampleannot,event=event,delay=delay,ratio=ratio)  
            g2iobject$nb3   <- tmp[[1]]
            g2iobject$txp3  <- tmp[[2]]
        }
       
        g2iobject$pnb3    <- sum(sampleannot$NB  <= g2iobject$nb3,na.rm=T)/length(na.exclude(sampleannot$NB ))
        g2iobject$ptxp3   <- sum(sampleannot$TXP <= g2iobject$txp3,na.rm=T)/length(na.exclude(sampleannot$TXP))
        
        sampleannot$G3   <- as.numeric(sampleannot[,"NB"] > g2iobject$nb3 & sampleannot[,"TXP"] > g2iobject$txp3)
      
        # step 5 >> get thresholds for group 1 (good prognosis)
        print("step 5")  
        
        if(method=="global"){
            noG3 <- which(sampleannot$G3==0)
            tmp <- g2i.getBestCutsG1(sampleannot[noG3,],sizeRange=c(10,sum(sampleannot$G3,na.rm=TRUE)+10),max.nb=g2iobject$nb3-1)
            w<-which.min(tmp[,4])
            g2iobject$nb1     <- as.numeric(tmp[w,"thresh.NB"])
            g2iobject$txp1    <- as.numeric(tmp[w,"thresh.TXP"])
        }else{
            tmp   <- g2i.learnThresholdsGroup1(sampleannot,event=event,delay=delay,ratio=ratio,nb3=g2iobject$nb3)
            g2iobject$nb1   <- tmp[[1]]
            g2iobject$txp1  <- tmp[[2]]
        }
        
        g2iobject$pnb1    <- sum(sampleannot$NB  <= g2iobject$nb1,na.rm=T)/length(na.exclude(sampleannot$NB ))
        g2iobject$ptxp1   <- sum(sampleannot$TXP <= g2iobject$txp1,na.rm=T)/length(na.exclude(sampleannot$TXP))        
        
        # step 6 >> calculate G2I groups
        print("step 6")
        g2iobject[[1]] <- g2i.subpredict( g2iobject=g2iobject, sampleannot=sampleannot)[[1]]
        
        attr(g2iobject,"method") <- method                             
   
        if (doplot){    
            g2i.plot(g2iobject,event=event,delay=delay)
        }
        
        g2iobject
  
}

# Rd
# description >> plot G2I results  
# argument
# item >> g2iresult >> list , with the same format as the result of a call to g2i.predict (or g2i.learn)
# item >> event >> ...
# item >> delay >>   ...
# value >> none
# author >> Aurelien de Reynies, Mickael Guedj
# keyword >> methods
# end 
g2i.plot <- function(g2iobject,horizontal=TRUE,event="relapse",delay="delay",cutAft=130,posLegend="bottomleft",posTest="center"){

        if(is(g2iobject)[1]=="list"){
            x <- g2iobject[[1]]
            x. <- c(g2iobject$txp1,g2iobject$txp3)
            y. <- c(g2iobject$nb1,g2iobject$nb3)
        }else{
            x <- g2iobject
            wG3 <- which(x$G2I==3) 
            wG1 <- which(x$G2I==1)
            y. <- c(max(x[wG1,"NB"]),min(x[wG3,"NB"])-1)
            x. <- c(max(x[wG1,"TXP"]),min(x[wG3,"TXP"])-.004) 
        }
        if(horizontal){
            par(mfrow=c(1,2))
        }else{
            par(mfrow=c(2,1))
        }
        
        plot(   x$TXP, 
                x$NB, 
                col = c("gray","black")[1+x[,event]],
                pch = 15,
                cex=.6, 
                ylab = "number of gained/lost regions",
                xlab = "% of gains / losses",
                main = "Prognosis x DNA copy aberration profile",
                cex.main=1)
        if(!is.null(x.) & !is.null(y.)) {
             rect( x.[2], y.[2], 1, 1000)
             rect(-1,-10,x.[1],y.[1])   
        }
        legend("bottomright",c("M-","M+"),fill=c("gray","black"))
                
        x$G2I <- paste("G2I-",x$G2I,sep="")  
        cit.surv(  x, 
                   event = event, 
                   delay = delay, 
                   strat = "G2I", 
                   cutDelayAfter =cutAft,
                   main = "Prognosis stratified per G2I group",
                   posLegend =posLegend, posTest=posTest,
                   ylab=paste("%",event),
                   strat.colors =function(z){c("green","blue","red")})
        NULL  
} 

################################################################################
#
#                        G2I : functions for internal use only
#
################################################################################

removeCentromericAndTelomericClones <- function(gnldata=NULL,
                                                cghannot=NULL,
                                                n=5,
                                                chrom = "chrom", 
                                                pos = "meanPos", 
                                                startPos = "start", 
                                                endPos = "end", 
                                                chromNum = "chrNum"){
    
    clones <- intersect(dimnames(cghannot)[[1]],dimnames(gnldata)[[1]])
    
    cghannot <- genomorder(cghannot[clones,],chrom=chrom,pos=pos,startPos=startPos,endPos=endPos,chromNum=chromNum)
    
    clonesOk <- unlist(sapply(unique(cghannot[,chrom]),function(chr){
                      xc <- rownames(cghannot)[which(cghannot[,chrom] == chr)]
                      return(xc[c((n+1):(length(xc)-(n+1)))])
            }))
    
    
    clonesOk
}

gainOrLossRate <- function(gnldata=NULL,cghannot=NULL,chromArm="arm"){
    clones <- intersect(dimnames(cghannot)[[1]],dimnames(gnldata)[[1]])
    Larm <- split(clones, cghannot[clones,chromArm])
    TXP0 <- as.data.frame(sapply(Larm, FUN = function(x){apply(gnldata[x,], 2, function(z) length(which(z != 0))/length(z))}))
    as.numeric(apply(TXP0, 1, FUN = mean))   
}

nbNonDiploidRegions <- function(gnldata=NULL,cghannot=NULL,chromArm="arm"){
    clones <- intersect(dimnames(cghannot)[[1]],dimnames(gnldata)[[1]])
    Larm <- split(clones, cghannot[clones,chromArm])
    gnlweight <- as.data.frame(apply(gnldata,2,function(z){
          z <- abs(z)
          try(z[which(z==0)] <- -3)
          try(z[which(is.na(z))] <- -1)
          z
    }))
    NB0 <- as.data.frame(sapply(Larm, FUN = function(x){apply(gnlweight[x,],MARGIN = 2,FUN = nbNonDiploidRegionsOnASegment)}))
    as.numeric(apply(NB0, 1, FUN = sum))
}

nbNonDiploidRegionsOnASegment <- function(x){
    if (max(x,na.rm=TRUE) > 0){
      sl = fastLocalScore(x)
      nb = dim(sl)[1]
    } else {
      nb = 0
    }
    nb
}

fastLocalScore <- function(x){   # x= vector GNL (0 -1 1) for a  chromos. arm for a sample        
        List = numeric(0)   
        k = 0
        L = 0
        R = 0
        for (i in 1:length(x)){
              L = R
              R = R+x[i]
              if (x[i] >= 0){              
                    List = rbind(List,c(i,i,L,R))
                    k = dim(List)[1]
                    cont = 1
                    while(cont == 1){                        
                          tmp <- subFastLocalScore(List)
                          cont <- tmp$cont
                          List <- tmp$List
                    }
              }
        }
        k <- length(List[,1])
        res <- matrix(0,k,3)
        for (i in 1:k) {
              res[i,] = c(List[i,1], List[i,2], List[i,4] - List[i,3])
        }
        rr <- order(res[,3], decreasing = TRUE)
        start <- res[rr,1]
        end <- res[rr,2]
        H <- res[rr,3]
        T. <- cumsum(H)
        data.frame(start, end, H, T.)
}

subFastLocalScore = function(List){
        cont <- 0
        k    <- 0
        if(is(List)[1] == "matrix"){
                k <- length(List[,1])   # nrow
        }
        if (is(List)[1] == "numeric" & length(List) > 0){
                k = 1
                List = t(as.matrix(List))
        }
        if (k >= 1){        
                index <- 1:k
                aa <- which(List[,3] < List[k,3])
                if (length(aa) > 0){                
                        j <- max(index[aa])
                        if(List[j,4] < List[k,4]){                        
                                cont <- 1
                                List[j,c(2,4)] <- List[k,c(2,4)]
                                List <- List[1:j,]
                        }
                }
        }
        res <- new.env()
        res$cont <- cont
        res$List <- List
        res
}








g2i.subpredict <- function( g2iobject=NULL, sampleannot=NULL){
    sampleannot$G2I <- 2
    sampleannot$G2I[which(sampleannot[,"NB"] > g2iobject$nb3 & sampleannot[,"TXP"] > g2iobject$txp3)] <- 3
    others <- which(sampleannot$G2I == 2)
    sampleannot[others[which(sampleannot[others,"NB"] < g2iobject$nb1 & sampleannot[others,"TXP"] < g2iobject$txp1)],"G2I"] <- 1         
    list(sampleannot=sampleannot,nb1=g2iobject$nb1,nb3=g2iobject$nb3,txp1=g2iobject$txp1,txp3=g2iobject$txp3,
         pnb1=g2iobject$pnb1,pnb3=g2iobject$pnb3,ptxp1=g2iobject$ptxp1,ptxp3=g2iobject$ptxp3)    
}

g2i.quantNormNBTXP <- function(cit,ext){

      ext$NBini  <- ext$NB
      ext$TXPini <- ext$TXP

      ok <- which(!is.na(ext$NB))
      nOk <- length(ok)
      ext$"NBnorm"<-NA
      ext[ok[order(ext$NB[ok])],"NBnorm"] <- round(quantile(cit$NB,probs=seq(.5, nOk - 0.5, by = 1)/nOk) )
  
      
      ok <- which(!is.na(ext$TXP))
      nOk <- length(ok)
      ext$"TXPnorm" <-NA
      ext[ok[order(ext$TXP[ok])],"TXPnorm"] <- round(quantile(cit$TXP,probs=seq(.5, nOk - 0.5, by = 1)/nOk) ,2)
 
      
      ext$NB  <- ext$NBnorm
      ext$TXP <- ext$TXPnorm
      
      ext
}



g2i.getBestCutsG3 <- function(x,event="relapse",delay="delay",ratio=.5){

      min.nb  <- min(x[,"NB"],na.rm=TRUE)
      max.nb  <- max(x[,"NB"],na.rm=TRUE)
      min.txp <- round(min(x[,"TXP"],na.rm=TRUE),2)
      max.txp <- round(max(x[,"TXP"],na.rm=TRUE),2)
  
      nEv <- sum(x[,event],na.rm=TRUE)
  
      v<- unique( x[,"NB"])
      nb <- (v[-1]+v[-length(v)])/2
      v<- unique( x[,"TXP"])
      tx <- (v[-1]+v[-length(v)])/2
     # nb <- .5+(min.nb:(max.nb-1))
     # tx <- seq(min.txp,max.txp,by=.005)
      u <- rep(nb,length(tx))
      v <- as.numeric(sapply(tx,function(z) rep(z,length(nb))))
      uv <- cbind("thresh.NB"=u,"thresh.TXP"=v)
      uv <- cbind(uv,"sizeG3"=as.numeric(sapply(1:nrow(uv),function(i)length(which( x$NB> uv[i,1] & x$TXP>uv[i,2])))))
      uv <- uv[which(uv[,"sizeG3"] >= ratio*nEv & uv[,"sizeG3"] <=nEv),]

      uv <- cbind(uv,"pvalLogrank"=as.numeric(sapply(1:nrow(uv),function(i){
                                          w<-which( x$NB> uv[i,1] & x$TXP>uv[i,2])
                                          strat <- numeric(nrow(x))
                                          strat[w] <- 1  
                                          pv <- NA  
                                          try(pv <- cit.surv(x, event = event, delay = delay, strat = strat, plot = FALSE)$strat[3,2],silent=TRUE)
                                          pv})))
     uv
}

g2i.getBestCutsG1 <- function(x,event="relapse",delay="delay",sizeRange=NULL,max.nb=NULL){

      min.nb  <- min(x[,"NB"],na.rm=TRUE)
      if(is.null(max.nb))max.nb  <- max(x[,"NB"],na.rm=TRUE)
      min.txp <- round(min(x[,"TXP"],na.rm=TRUE),2)
      max.txp <- round(max(x[,"TXP"],na.rm=TRUE),2)
  
      v<- unique( x[,"NB"])
      nb <- (v[-1]+v[-length(v)])/2
      v<- unique( x[,"TXP"])
      tx <- (v[-1]+v[-length(v)])/2
  
   #   nb <- .5+(min.nb:(max.nb-1))
   #   tx <- seq(min.txp,max.txp,by=.005)
      u <- rep(nb,length(tx))
      v <- as.numeric(sapply(tx,function(z) rep(z,length(nb))))
      uv <- cbind("thresh.NB"=u,"thresh.TXP"=v)
      uv <- cbind(uv,"sizeG1"=as.numeric(sapply(1:nrow(uv),function(i)length(which( x$NB< uv[i,1] & x$TXP<uv[i,2])))))
      uv <- uv[which(uv[,"sizeG1"] >= sizeRange[1] & uv[,"sizeG1"] <=sizeRange[2]),]

      uv <- cbind(uv,"pvalLogrank"=as.numeric(sapply(1:nrow(uv),function(i){
                                          w<-which( x$NB< uv[i,1] & x$TXP<uv[i,2])
                                          strat <- numeric(nrow(x))
                                          strat[w] <- 1  
                                          pv <- NA  
                                          try(pv <- cit.surv(x, event = event, delay = delay, strat = strat, plot = FALSE)$strat[3,2],silent=TRUE)
                                          pv})))
     uv
}






g2i.learnThresholdsGroup3 <- function(sampleannot,event="relapse",delay="delay",ratio=0.5){

      nnb   <- NULL
      ttxp  <- NULL
      ppv   <- NULL
      
      min.nb  <- min(sampleannot[,"NB"],na.rm=TRUE)
      max.nb  <- max(sampleannot[,"NB"],na.rm=TRUE)
      min.txp <- min(sampleannot[,"TXP"],na.rm=TRUE)
      max.txp <- max(sampleannot[,"TXP"],na.rm=TRUE)
                                                                                                   
      for (i in 1:500){
            RES <- try(  optim(  par = c(runif(1,min.nb , max.nb/2), runif(1,min.txp,max.txp/2)), g2i.getScoreThresholdsGroup3, sampleannot = sampleannot, ratio = ratio,event=event), silent = TRUE)
            if (!is.na(RES[2])){                  
                  nnb[i]   <- RES$par[1]
                  ttxp[i]  <- RES$par[2]
                  strat    <- rep(0, dim(sampleannot)[1])
                  w        <- which(sampleannot[,"NB"] > RES$par[1] & sampleannot[,"TXP"] > RES$par[2])
                  strat[w] <- 1
                  res      <- cit.surv(sampleannot, event = event, delay = delay, strat = strat, plot = FALSE)
                  ppv[i]   <- res$strat[3,2]    # <-> logrank pvalue
            }
      }
    
      wbestpval <- which.min(ppv)
      nb3       <- nnb[wbestpval]
      txp3      <- ttxp[wbestpval]      
      pnb3      <- sum(sampleannot$NB  <= nb3,na.rm=T)/length(na.exclude(sampleannot$NB ))
      ptxp3     <- sum(sampleannot$TXP <= txp3,na.rm=T)/length(na.exclude(sampleannot$TXP))

      list(nb3=nb3,txp3=txp3,pnb3=pnb3,ptxp3=ptxp3)
}

g2i.getScoreThresholdsGroup3 <- function(x,#  <-> c(nb, txp) 
                                         sampleannot, 
                                         ratio = 0.5,
                                         event="relapse"){
                    
    if (x[1] < min(sampleannot[,"NB"]) | x[2] < min(sampleannot[,"TXP"]) | x[1] > max(sampleannot[,"NB"]) | x[2] > max(sampleannot[,"TXP"])){
      return(NA)
    }
    
    wG3   <- which(sampleannot[,"NB"] > x[1] & sampleannot[,"TXP"] > x[2])
    wEv   <- which(sampleannot[,event] == 1) 
    wG3Ev <- intersect(wG3,wEv)
    
    nG3Ev <- length(wG3Ev)       
    nG3   <- length(wG3)     
    nEv   <- length(wEv)  
    
    if(nG3 < nEv*ratio | nG3 > nEv){   # constraint on size of G3 
      return(NA)
    }
    Spe <- nG3Ev/nG3
    Sen <- nG3Ev/nEv  
    return(1-Spe*Sen)
}



g2i.learnThresholdsGroup1 <- function(sampleannot,event="relapse",delay="delay",ratio=0.5,nb3=NULL){

      others <- which(sampleannot$G3 == 0)
      
      nnb   <- NULL
      ttxp  <- NULL
      ppv   <- NULL

      min.nb  <- min(sampleannot[,"NB"],na.rm=TRUE)
      max.nb  <- max(sampleannot[,"NB"],na.rm=TRUE)
      min.txp <- min(sampleannot[,"TXP"],na.rm=TRUE)
      max.txp <- max(sampleannot[,"TXP"],na.rm=TRUE)
   
      for (i in 1:500){
    
        RES = try(optim(par = c(runif(1, min.nb, max.nb/2), runif(1,min.txp,max.txp/2)), g2i.getScoreThresholdsGroup1, sampleannot = sampleannot[others, ], event=event,ratio=ratio), silent = TRUE)
        if (!is.na(RES[2])){
            nnb[i]    <- RES$par[1]
            ttxp[i]   <- RES$par[2]
            strato    <- rep(1, length(others))
            w         <- which(sampleannot[others,"NB"] < RES$par[1] & sampleannot[others,"TXP"] < RES$par[2])
            strato[w] <- 0
            res       <- cit.surv(sampleannot[others, ], event = event, delay = delay, strat = strato, plot = FALSE)
            ppv[i]    <- res$strat[3,2]    # <-> logrank pvalue     
            if(nnb[i] >=  nb3 ) ppv[i]<-1  
        }
      }
    
      wbestpval <- which.min(ppv)
      nb1       <- nnb[wbestpval]
      txp1      <- ttxp[wbestpval]  
      pnb1      <- sum(sampleannot$NB < nb1,na.rm=T)/length(na.exclude(sampleannot$NB))
      ptxp1     <- sum(sampleannot$TXP < txp1,na.rm=T)/length(na.exclude(sampleannot$TXP))

      list(nb1=nb1,txp1=txp1,pnb1=pnb1,ptxp1=ptxp1)
}
  

g2i.getScoreThresholdsGroup1 <- function( x,# = c(nb, txp), 
                                          sampleannot, 
                                          event="relapse",
                                          ratio=.5){
                                          
    if (x[1] < min(sampleannot[,"NB"]) | x[2] < min(sampleannot[,"TXP"]) | x[1] > max(sampleannot[,"NB"]) | x[2] > max(sampleannot[,"TXP"])){
      return(NA)
    }
    
    wG1   <- which(sampleannot[,"NB"] < x[1] & sampleannot[,"TXP"] < x[2])  #)# 
    wNoEv   <- which(sampleannot[,event] == 0) 
    wG1NoEv <- intersect(wG1,wNoEv)
    
    nG1NoEv <- length(wG1NoEv)       
    nG1   <- length(wG1)     
    nNoEv   <- length(wNoEv)  
    nEv <- sum(sampleannot[,event],na.rm=TRUE)
    if(nG1>nEv | nG1<nEv*ratio)return(NA)

    Spe <- nG1NoEv/nG1
    Sen <- nG1NoEv/nNoEv  
    return(1-Spe*Sen)                                      
}


#changeRange  <- function (v, newmin = 1, newmax = 10) 
#{
#    oldmin <- min(v, na.rm = TRUE)
#    oldmax <- max(v, na.rm = TRUE)
#    newmin + ((newmax - newmin) * (v - oldmin)/(oldmax - oldmin))
#}
#
#normDistrib <- function(ref,other){
#      refmean <- mean(ref,na.rm=TRUE)
#      refq4999 <- quantile(ref,probs=.4999,na.rm=TRUE)
#      refmin  <- min(ref,na.rm=TRUE)
#      refmax  <- max(ref,na.rm=TRUE) 
#      othermean <- mean(other,na.rm=TRUE)
#      wol <- which(other < othermean)
#      wor <- which(other >= othermean)
#      other[wol] <- changeRange(other[wol],refmin,refq4999)
#      other[wor] <- changeRange(other[wor],refmean,refmax)
#      other
#}